/*
 * Copyright 2015 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import Ubuntu.Components 1.1
import Ubuntu.Components.Popups 0.1
import Ubuntu.Content 0.1
import Ubuntu.OnlineAccounts.Client 0.1
import Ubuntu.PushNotifications 0.1
import UserMetrics 0.1
import U1db 1.0 as U1db

import QtQuick 2.0
import QtMultimedia 5.0
import Qt.labs.settings 1.0

import Telegram 0.1
import "js/version.js" as Version
import "ui"
import "components"

MainView {
    property bool initCalled: false
    property bool connected: false

    property alias telegramClient: telegramClientLoader.item
    property alias pushClient: pushClientLoader.item
    property alias pushDialog: pushDialogLoader.item

    property string appId: "com.ubuntu.telegram_telegram" // no-i18n
    property bool promptForPush:
            (typeof appSettings.contents.promptForPush === "undefined")
            ? false: appSettings.contents.promptForPush

    property var uri: undefined
    property int chatToOpen: 0
    property var chat: undefined

    id: root
    // For autopilot-qt4 functional testing.
    objectName: "mainView"

    // Must match 'name' in click manifest.
    applicationName: "com.ubuntu.telegram"

    automaticOrientation: false

    useDeprecatedToolbar: false

    anchorToKeyboard: true

    width: units.gu(45)
    height: units.gu(80)

    onActiveFocusChanged: {
        if (initCalled) {
            if (activeFocus) {
                processUri();
                console.log("Telegram resuming..");
                telegramClient.wake();
            } else {
                console.log("Telegram suspending..");
                telegramClient.sleep();
            }
        }

        if (activeFocus) {
            if (pushClientLoader.status === Loader.Ready) {
                // We do this to register for push when U1 account was
                // set-up after Telegram was run without it first.
                if (!pushClient.registered) {
                    console.log("push - retrying registration");
                    pushClient.appId = "push-bug-workaround";
                    pushClient.appId = root.appId;
                } else {
                    // App brought to foreground. Dismiss notifications.
                    if (pageStack.currentPage.objectName === "chatPage") {
                        var chatId = pageStack.currentPage.chatId;
                        pushClient.clearPersistent([chatId]);
                    }
                }
            }
        }
    }

    Connections {
        target: UriHandler
        onOpened: {
            root.uri = uris[0];
            console.log("uri opened: " + root.uri);
            processUri();
        }
    }

    Setup {
        id: setup
        applicationId: "com.ubuntu.telegram_telegram" // no-i18n
        providerId: "ubuntuone" // no-i18n
    }

    Item {
        id: transferHelper
        property variant transfer
        property string type: "unknown" // no-i18n
        property int count: 0
        property bool hasContent: count > 0

        property string message
        property string pathToFile

        function setup(transfer) {
            transferHelper.transfer = transfer;

            count = transfer.items.length;
            message = "";
            pathToFile = "";
            switch (transfer.contentType) {
            case ContentType.Pictures:
                transferHelper.type = "pictures"; // no-i18n
                break;
            case ContentType.Videos:
                transferHelper.type = "videos"; // no-i18n
                break;
            case ContentType.Contacts:
            case ContentType.Documents:
            case ContentType.Music:
            case ContentType.Unknown:
                transferHelper.type = "documents"; // no-i18n
                break;
            case ContentType.Links:
                transferHelper.type = "url"; // no-i18n
                break;
            case ContentType.Text:
                transferHelper.type = "text"; // no-i18n
                break;
            default:
                console.log("Transfer has undefined content type!");
                transferHelper.type = "documents"; // no-i18n
                break;
            }
        }

        function cancel() {
            if (typeof transfer === "undefined") {
                console.log("Can't cancel non-existent transfer!");
                return;
            }
            transfer.state = ContentTransfer.Aborted;
            reset();
        }

        function reset() {
            type = "unknown"; // no-i18n
            count = 0;
        }
    }

    Connections {
        target: ContentHub
        onShareRequested: {
            backToDialogsPage();
            processTransfer(transfer)
        }
    }

    function startsWith(string, prefix) {
        return string.indexOf(prefix) === 0;
    }

    function processTransfer(transfer) {
        transferHelper.setup(transfer);

        var url = String(transfer.items[0].url);
        if (startsWith(url, "file://")) {
            transferHelper.pathToFile = url.replace('file://', '');
            console.log ("Received file transfer");
        } else {
            var text = String(transfer.items[0].text);
            if (url.length > 0) {
                if (text > 0) {
                    transferHelper.message = text + ": " + url; // no-i18n
                } else {
                    transferHelper.message = url;
                }
            } else {
                transferHelper.message = text;
            }
            console.log ("Received text/url transfer");
        }
    }

    QtObject {
        id: userData
        property string phoneNumber: ""
        property string firstName: ""
        property string lastName: ""

        function reset() {
            phoneNumber = "";
            firstName = "";
            lastName = "";
        }
    }

    Metric {
        id: incomingMessagesMetric
        name: "tg-incoming-msgs" // no-i18n
        format: i18n.tr("Telegram messages received today: %1")
        emptyFormat: i18n.tr("No Telegram messages received today.")
        domain: applicationName
    }

    Metric {
        id: outgoingMessagesMetric
        name: "tg-outgoing-msgs" // no-i18n
        format: i18n.tr("Telegram messages sent today: %1")
        emptyFormat: i18n.tr("No Telegram messages sent today.")
        domain: applicationName
    }

    PageStack {
        id: pageStack

        // --- intro page
        PageLoader {
            id: introPageLoader
            sourceComponent: IntroPage {
                id: introPage
            }
            onLoaded: pageStack.showIntroPage()
        }

        Connections {
            ignoreUnknownSignals: true
            target: introPageLoader.valid ? introPageLoader.item : null
            onShowSignInPage: pageStack.showSignInPage()
        }

        function showIntroPage() {
            if (introPageLoader.status === Loader.Ready) {
                pageStack.clear();
                pageStack.push(introPageLoader.item);
            } else {
                introPageLoader.active = true;
            }
        }
        // --- end of: intro page

        function showSignInPage() {
            pageStack.push(signInPage);
            appSettings.set("isFirstLaunch", false);
        }

        Component {
            id: signInPage
            SignInPage {}
        }

        Component {
            id: picker
            PickerPage {}
        }

        Component {
            id: signUpPage
            SignUpPage {}
        }

        Component {
            id: verificationCodePage
            VerificationCodePage {
                objectName: "verificationPage"
            }
        }

        Component {
            id: dialogsPage
            DialogsPage {
                objectName: "dialogsPage"
            }
        }

        Component {
            id: dialogPage
            DialogPage {
                objectName: "chatPage"
            }
        }

        Component {
            id: profilePage
            ProfilePage {}
        }

        Component {
            id: groupProfilePage
            GroupProfilePage {
                objectName: "groupProfilePage"
            }
        }

        Component {
            id: contactsPage
            ContactsPage {}
        }

        Component {
            id: addContactPage
            AddContactPage {}
        }

        Component {
            id: settingsPage
            SettingsPage {}
        }

        Component {
            id: previewPage
            PreviewPage {}
        }

        Component {
            id: editProfilePage
            EditProfilePage {}
        }

        Component {
            id: blockedUsersPage
            BlockedUsersPage {}
        }

        function onFirstLaunch() {
            showIntroPage();
        }

        function onLaunch() {
            pageStack.clear();
            pageStack.push(signInPage);
        }

        function onLoggedInLaunch() {
            pageStack.clear();
            pageStack.push(dialogsPage);

            var phone = appSettings.contents.phoneNumber;
            console.log("reusing phone number: " + phone);
            setupTelegram(phone);
        }
    }

    function setupTelegram(phoneNumber) {
        telegramClient.phoneNumber = phoneNumber;
        telegramClient.init();
        initCalled = true;
    }

    function backToDialogsPage() {
        while (pageStack.depth > 0 &&
               pageStack.currentPage.objectName !== "dialogsPage") {
            pageStack.pop();
        }
        if (pageStack.depth === 0) {
            pageStack.push(dialogsPage);
        }
    }

    function openChatById(chatId, properties) {
        if (pageStack.depth > 0 && pageStack.currentPage.objectName === "chatPage") {
            if (pageStack.currentPage.chatId === chatId) return;
        }
        if (typeof properties === "undefined") properties = { };
        backToDialogsPage();

        properties['chatId'] = chatId;
        pageStack.push(dialogPage, properties);

        pushClient.clearPersistent([chatId]);
    }

    function newChat() {
        openContacts(false);
    }

    function newGroupChat() {
        openContacts(true);
    }

    function openProfile(dialogId) {
        var properties = { dialogId: dialogId };
        pageStack.push(profilePage, properties)
    }

    function openGroupProfile(chatId) {
        var properties = { chatId: chatId };
        pageStack.push(groupProfilePage, properties)
    }

    function openContacts(groupChatMode) {
        var properties = { groupChatMode: groupChatMode };
        pageStack.push(contactsPage, properties);
    }

    function openSettings() {
        pageStack.push(settingsPage);
    }

    function showAddContactPage(addManually, peerId, contact) {
        var properties = { addManually: addManually, peerId: peerId };
        if (contact !== undefined) {
            properties["contact"] = contact; // no-i18n
        }
        pageStack.push(addContactPage, properties);
    }

    function forwardMessages(messages) {
        var properties = { messagesToForward: messages };
        pageStack.push(dialogsPage, properties);
    }

    function doUpgradeToDatabase() {
        if (appSettings.contents.loadFromSettings) {
            console.log("Upgrading from Settings to U1db.Database")
            appSettings.set("isFirstLaunch", appSettingsLegacy.isFirstLaunch);
            appSettings.set("isLoggedIn", appSettingsLegacy.isLoggedIn);
            appSettings.set("phoneNumber", appSettingsLegacy.phoneNumber);
            appSettings.set("promptForPush", appSettingsLegacy.promptForPush);
            appSettings.set("loadFromSettings", false);
        }
    }

    Settings {
        // TODO Update Qt.application.domain to fix this path:
        // ~/.config/qt-project.org/com.ubuntu.telegram.conf
        id: appSettingsLegacy
        category: "session" // no-i18n

        property bool isFirstLaunch: true
        property bool isLoggedIn: false
        property string phoneNumber: ""
        property bool promptForPush: true
    }

    U1db.Database {
        id: database
        // TODO: change /home/phablet to ~ as soon as bug #1387294 is fixed
        path: "/home/phablet/.config/%1/settings.u1db".arg(applicationName)
    }

    U1db.Document {
        id: appSettings
        database: database
        docId: "appSettings"
        defaults: {
            "loadFromSettings": true,   // no-i18n
            "isFirstLaunch": true,      // no-i18n
            "isLoggedIn": false,        // no-i18n
            "phoneNumber": "",          // no-i18n
            "promptForPush": true       // no-i18n
        }
        create: true

        function set(key, value) {
            var tmp = contents;
            tmp[key] = value;
            contents = tmp;

            if (key === "phoneNumber") {
                // remove spaces and +
                userSettings.setNumber(value);
            }
        }

        Component.onCompleted: {
            // WORKAROUND: it seems that when the object is created, the contents aren't
            // still set. We force the update of its contents
            if (database.listDocs().indexOf(docId) < 0) {
                contents = defaults;
            }
            doUpgradeToDatabase();
        }

    }

    U1db.Document {
        id: userSettings
        database: database
        create: false
        defaults: { "sendByEnter": false, "chatBackground": "", "fontSize": "medium" }

        function set(key, value) {
            var tmp = contents;
            tmp[key] = value;
            contents = tmp;
        }

        function setNumber(phoneNumber) {
            docId = Qt.md5(phoneNumber.replace(/[\s\+\(\)]/g, ''));
            if (database.listDocs().indexOf(docId) < 0) {
                contents = defaults;
            }
        }

        Component.onCompleted: {
            if (typeof appSettings.contents.phoneNumber !== "undefined") {
                setNumber(appSettings.contents.phoneNumber);
            }
        }
    }

    Loader {
        id: pushClientLoader
        active: false
        asynchronous: false
        sourceComponent: PushClient {
            id: pushClient
            appId: root.appId

            property bool connected: telegramClient.connected
            property bool registered: false

            onTokenChanged: {
                console.log("push - token changed: " + (token.length > 0 ? "present" : "empty!"));
                if (token.length > 0) {
                    registered = false;
                    registerForPush();
                }
            }

            onError: {
                if (status == "bad auth") {
                    console.warn("push - 'bad auth' error: " + status);
                    if (promptForPush) {
                        pushDialogLoader.active = true;
                    }
                } else {
                    console.warn("push - unhandled error: " + status);
                }
            }

            onConnectedChanged: {
                if (token.length > 0) {
                    registerForPush();
                }
            }

            function registerForPush() {
                if (registered) return;

                if (token.length === 0) {
                    console.warn("push - can't register, empty token.");
                    return;
                }
                if (!connected) {
                    console.warn("push - can't register, not connected yet.");
                    return;
                }
                if (!appSettings.contents.isLoggedIn) {
                    console.warn("push - can't register, not logged-in yet.");
                    return;
                }

                console.log("push - registering with Telegram now");
                telegramClient.accountRegisterDevice(token, Version.version);
            }

            function unregisterFromPush() {
                if (registered) {
                    telegramClient.accountUnregisterDevice(token);
                } else {
                    console.warn("push - could not unregister from push!");
                }
            }
        }

        onStatusChanged: {
            if (status === Loader.Loading) {
                console.log("push - push client loading");
            } else if (status === Loader.Ready) {
                console.log("push - push client loaded");
            }
        }

        onLoaded: {
            pushClient.notificationsChanged.connect(telegramClient.onPushNotifications);
            pushClient.error.connect(telegramClient.onPushError);

            if (!appSettings.contents.isLoggedIn) {
                // This should never happen, as we start the Loader in onAuthLoggedIn.
                console.log("push - loaded pushClient, but not logged in!");
                return;
            }
        }
    }

    Loader {
        id: telegramClientLoader
        active: false
        asynchronous: false
        sourceComponent: TelegramClient {
            id: telegramClient
            configPath: "~/.config/" + applicationName
            publicKeyFile: "telegram.pub" // no-i18n
            property real lastCheckedId: 0

            function logoutResetSession(showIntro) {
                appSettings.set("isLoggedIn", false);
                appSettings.set("phoneNumber", "");
                appSettings.set("isFirstLaunch", showIntro);
                initCalled = false;
            }

            function logoutResetUI(showIntro) {
                pageStack.clear();
                if (showIntro) {
                    pageStack.showIntroPage();
                } else {
                    pageStack.push(signInPage);
                }
            }

            function logout(showIntro) {
                if (typeof showIntro === "undefined") {
                    showIntro = false;
                }

                pushClient.count = 0;
                pushClient.clearPersistent([]);

                logoutResetSession(showIntro);
                logoutResetUI(showIntro);
            }

            onError: {
                var msg = errorCode + " " + errorText;
                if (errorCode < 500) {
                    console.warn("Client error: " + msg);
                } else {
                    console.warn("Server error: " + msg);
                }

                if (errorCode === 401) {
                    if (errorText === "SESSION_REVOKED") { // no-i18n
                        telegramClient.logout(true);
                    } else {
                        // We want to reset session now. When app is re-launched
                        // manually, it should not be signed in already.
                        telegramClient.logoutResetSession();
                        openLoggedOutDialog();
                    }
                }
            }

            onFatalError: {
                logout();
            }

            onAuthNeeded: {
                authCheckPhone();
            }

            onAuthCheckPhoneSent: {
                if (telegramClient.phoneNumber === phoneNumber) {
                    telegramClient.lastCheckedId = msgId;
                }
            }

            onAuthCheckPhoneAnswer: {
                if (id !== lastCheckedId) return;
                if (phoneRegistered) {
                    authSendCode();
                } else {
                    pageStack.push(signUpPage);
                }
            }

            onAuthLoggedIn: {
                if (!appSettings.contents.isLoggedIn) {
                    appSettings.set("isLoggedIn", true);
                    appSettings.set("phoneNumber", userData.phoneNumber);
                    appSettings.set("isFirstLaunch", false);
                }

                if (pageStack.currentPage.objectName === "verificationPage") {
                    pageStack.clear();
                    pageStack.push(dialogsPage);
                }

                dialogsModel.setTelegramClient(telegramClient);

                if (!pushClientLoader.active) {
                    pushClientLoader.active = true;
                } else if (pushClientLoader.status === Loader.Ready) {
                    pushClient.registerForPush();
                }
            }

            onAuthLogOutAnswer: {
                logout(!ok);
            }

            onAuthResetAuthorizationsAnswer: {
                console.log("terminated all other sessions");
                if (ok) {
                    console.log("registering again for push notifications");
                    pushClient.registered = false;
                    pushClient.registerForPush();
                }
            }

            onAccountRegisterDeviceAnswer: {
                if (ok) {
                    console.log("push - registered with Telegram");
                } else {
                    console.warn("push - failed to register with Telegram!");
                }
                pushClient.registered = ok;
            }

            onAccountUnregisterDeviceAnswer: {
                if (ok) {
                    console.log("push - unregistered from Telegram");
                } else {
                    console.warn("push - failed to unregister from Telegram!");
                }
                // Set unregistered state regardless if it succeeded.
                pushClient.registered = false;
            }

            onWoken: {
                /*
                  Sometimes the app does not get activeFocus when
                  it is brought to foreground. This strange behavior
                  must be a platform bug. A side effect is opening
                  the requested chat the *next* time the app is
                  brought to foreground.
                */

                telegramClient.chatToOpen = root.chatToOpen;

                if (pushClient.registered) {
                    pushClient.getNotifications();
                }
            }

            onUnreadCountChanged: {
                pushClient.count = unreadCount;
                if (unreadCount === 0) {
                    pushClient.clearPersistent([]);
                }
            }

            onMessageSent: outgoingMessagesMetric.increment()
            onMessageReceived: incomingMessagesMetric.increment()

            // We need to clean-up and streamline our APIs ;)

            onOpenChat: {
                root.chatToOpen = 0;
                root.openChatById(id);
            }

            onMessagesCreateChatAnswer: {
                root.openChatById(chat.id);
            }

            onSecretChatCreated: {
                // Until we know who created the chat, don't pop it open.
                // root.openChatById(chatId);
            }
        }

        onLoaded: {
            telegramClient.chatToOpen = root.chatToOpen;
            if (appSettings.contents.isFirstLaunch) {
                pageStack.onFirstLaunch();
            } else if (!appSettings.contents.isLoggedIn) {
                pageStack.onLaunch();
            } else {
                pageStack.onLoggedInLaunch();
            }
        }
    }

    DialogsModel {
        id: dialogsModel
    }

    function openPushDialog() {
        // karni: Consider moving this to userSettings instead.
        appSettings.set("promptForPush", true);
        if (pushDialogLoader.status === Loader.Ready) {
            PopupUtils.open(pushDialogComponent);
        } else {
            pushDialogLoader.active = true;
        }
    }

    Loader {
        id: pushDialogLoader
        active: false
        asynchronous: false
        sourceComponent: Component {
            id: pushDialogComponent
            Dialog {
                id: pushDialog
                title: i18n.tr("Notifications")
                text: i18n.tr("If you want Telegram notifications when you're not using the app, sign in to your Ubuntu One account, then switch back to Telegram.")
                Button {
                    text: i18n.tr("Sign in to Ubuntu One")
                    color: UbuntuColors.orange
                    onClicked: {
                        setup.exec();
                        close();
                    }
                }
                Button {
                    text: i18n.tr("Remind me later")
                    onClicked: close()
                }
                Button {
                    text: i18n.tr("Don't want notifications")
                    onClicked: {
                        appSettings.set("promptForPush", false);
                        pushClient.unregisterFromPush();
                        close();
                    }
                }

                function close() {
                    PopupUtils.close(pushDialog);
                }
            }
        }

        onLoaded: {
            openPushDialog();
        }
    }

    function openLoggedOutDialog() {
        if (loggedOutDialogLoader.status === Loader.Ready) {
            PopupUtils.open(loggedOutDialogComponent);
        } else {
            loggedOutDialogLoader.active = true;
        }
    }

    Loader {
        id: loggedOutDialogLoader
        active: false
        asynchronous: false
        sourceComponent: Component {
            id: loggedOutDialogComponent
            Dialog {
                id: loggedOutDialogDialog
                title: i18n.tr("Unauthorized")
                text: i18n.tr("You have been logged out. Please log in.")
                Button {
                    text: i18n.tr("OK")
                    color: UbuntuColors.orange
                    onClicked: {
                        telegramClient.logout();
                        close();
                    }
                }

                function close() {
                    PopupUtils.close(loggedOutDialogDialog);
                }
            }
        }

        onLoaded: {
            openLoggedOutDialog();
        }
    }

    Component.onCompleted: {
        if (typeof uriArgs !== "undefined" && uriArgs) {
            root.uri = uriArgs[0];
            processUri();
        }

        telegramClientLoader.active = true;
    }

    function processUri() {
        if (typeof root.uri === "undefined") return;
        var commands = root.uri.split("://")[1].split("/");
        if (commands) {
            switch(commands[0].toLowerCase()) {
            case "chat": // no-i18n
                var chatId = parseInt(commands[1]);
                if (isNaN(chatId) || chatId === 0) {
                    console.info("Not setting chat to open.");
                    backToDialogsPage();
                    // TODO: Temporarily open no chats if secret notification pressed.
                } else {
                    console.info("Setting chat to open: " + chatId);
                    root.chatToOpen = chatId;
                    root.uri = undefined;
                }
            case "launch": // no-i18n
                // Nothing to do.
                break;
            default: console.warn("Unmanaged URI! " + commands);
            }
        }
        commands = undefined;
    }
}
