import QtQuick 2.0
import QtMultimedia 5.0
import Ubuntu.Components 1.1
import Ubuntu.Components.Popups 0.1
import Ubuntu.Components.ListItems 1.0 as ListItems
import Ubuntu.Content 1.0
import Telegram 0.1

import "../components"
import "../components/TelegramColors.js" as TelegramColors
import "../js/avatar.js" as Avatar
import "../js/time.js" as Time

TelegramPage {
    id: dialogPage

    property bool isChat: dialogModel.isChat
    property int secretChatState: dialogModel.state
    property bool actionsEnabled: pageIsSecret ? secretChatState === 2 : true

    property bool hasPhone: dialogModel.phone !== ""
    property bool isForeign: dialogModel.peerType === TLInputPeer.TypeInputPeerForeign

    property bool shareContent: false
    property var messagesToForward: []

    title: dialogModel.title
    pageImage: dialogModel.thumbnail

    pageIsSecret: dialogModel.isSecret
    /*
    // TODO dialogItem.lastSeenOnline does not work.
    pageSubtitle: {
        var lastSeenOnline = dialogItem.lastSeenOnline;
        if (lastSeenOnline !== 0) {
            var lastSeenString = Time.formatLastSeen(i18n, lastSeenOnline * 1000);
            return i18n.tr("last seen %1").arg(lastSeenString);
        } else {
            return "";
        }
    }
    */

    pageSubtitleAlt: {
        var typingMsg = "";
        if (dialogModel.isTyping) {
            if (dialogModel.isChat) {
                typingMsg = i18n.tr("%1 is typing...").arg(dialogModel.whoIsTyping);
            } else {
                typingMsg = typingMsg += i18n.tr("typing...");
            }
        }
        return typingMsg;
    }

    onHeaderClicked: {
        Qt.inputMethod.hide();

        var userId = pageIsSecret ? dialogModel.peerId : dialogModel.chatId;
        isChat ? openGroupProfile(dialogModel.chatId)
               : openProfile(dialogModel.chatId);
    }

    signal addContactsAnswer(var contactIds);
    signal secretChatCreated(int chatId);
    signal secretChatDiscarded(int chatId);

    DialogModel {
        id: dialogModel

        onIsSecretChanged: {
            // Forward to page header.
            pageIsSecret = dialogModel.isSecret;
        }

        onTitleChanged: {
            // Forward to page header.
            pageTitle = dialogModel.title;
        }

        onTtlChanged: {
            if (!pageIsSecret) return;

            if (ttl > 0) {
                pageSubtitle = i18n.tr("Self destruct ") +
                        delegateUtils.getSecretChatSelfDestructTime(dialogModel.ttl);
            } else {
                pageSubtitle = "";
            }
        }

        onDialogDetailsRefreshed: {
            console.log("Chat details refreshed.");
        }
    }

    MessagesModel {
        id: messagesModel

        dialogId: dialogModel.chatId
        peerType: dialogModel.peerType
        isSecret: dialogModel.isSecret

        onCountIncreased: {
            // Mark visible messages as read.
            if (pageIsSecret) {
                telegramClient.readSecretChatHistory(dialogId);
            } else {
                telegramClient.messagesReadHistory(inputPeer);
            }
        }
    }

    TLInputPeer {
        id: inputPeer

        userId: dialogModel.isSecret ? 0 :
                    (dialogModel.isChat ? 0 : dialogModel.chatId)
        chatId: dialogModel.isSecret ? 0 :
                    (dialogModel.isChat ? dialogModel.chatId : 0)
        accessHash: dialogModel.accessHash
        classType: dialogModel.peerType
    }


    Component.onCompleted: {
        dialogModel.setup(telegramClient, chatId);

        messagesModel.setTelegramClient(telegramClient);
        messagesModel.refresh();

        // TODO Connect those signals in C++ instead.
        telegramClient.addContactsAnswer.connect(dialogPage.addContactsAnswer);
        telegramClient.secretChatCreated.connect(dialogPage.secretChatCreated);
        telegramClient.secretChatDiscarded.connect(dialogPage.secretChatDiscarded);

        if (transferHelper.hasContent && shareContent) {
            if (transferHelper.type === "pictures") {
                sendPhoto(transferHelper.pathToFile);
            } else if (transferHelper.type === "videos") {
                sendVideo(transferHelper.pathToFile);
            } else if (transferHelper.type === "documents") {
                sendDocument(transferHelper.pathToFile);
            } else if (transferHelper.type === "text" || transferHelper.type === "url") {
                sendMessage(transferHelper.message);
            } else {
                transferHelper.cancel();
            }
            transferHelper.reset();
        }

        if (dialogPage.messagesToForward.length > 0) {
            telegramClient.messagesForwardMessages(inputPeer, messagesToForward);
        }
    }

    Component.onDestruction: {
        telegramClient.addContactsAnswer.disconnect(dialogPage.addContactsAnswer);
        telegramClient.secretChatCreated.disconnect(dialogPage.secretChatCreated);
        telegramClient.secretChatDiscarded.disconnect(dialogPage.secretChatDiscarded);
    }

    property list<Action> defaultActions: [
        Action {
            iconName: "stock_contact"
            text: isChat ? i18n.tr("Group Info") : i18n.tr("Profile Info")
            onTriggered: {
                Qt.inputMethod.hide();
                headerClicked();
            }
        },
        Action {
            iconName: "stock_alarm-clock"
            text: i18n.tr("Set self-destruct timer")
            enabled: isConnected && actionsEnabled
            visible: pageIsSecret && secretChatState === 2
            onTriggered: {
                Qt.inputMethod.hide();
                showSelfDestructDialog();
            }
        },
        Action {
            iconName: "clear"
            text: i18n.tr("Clear history")
            enabled: isConnected && actionsEnabled
            onTriggered: {
                Qt.inputMethod.hide();
                showClearHistoryDialog();
            }
        }
    ]

    property list<Action> selectionActions: [
        Action {
            iconName: "select"
            text: i18n.tr("Select all")
            onTriggered: {
                if (list.selectedItems.count === list.listModel.count) {
                    list.clearSelection()
                } else {
                    list.selectAll()
                }
            }
        },
        Action {
            id: copySelectedAction
            iconName: "edit-copy"
            text: i18n.tr("Copy")
            onTriggered: list.copySelected()
        },
        Action {
            id: forwardSelectedAction
            iconName: "next"
            text: i18n.tr("Forward")
            visible: !pageIsSecret
            onTriggered: list.forwardSelected()
        },
        Action {
            id: multiDeleteAction
            iconName: "delete"
            text: i18n.tr("Delete")
            onTriggered: list.deleteSelected()
        }
    ]

    head.actions: list.isInSelectionMode ? selectionActions : defaultActions

    isInSelectionMode: list.isInSelectionMode
    onSelectionCanceled: list.cancelSelection()

    body: Item {

        anchors {
            fill: parent
        }

        Image {
            anchors.fill: parent
            source: Qt.resolvedUrl("../images/background_hd.jpg");
            fillMode: Image.Center

            Component.onCompleted: {
                // Set size here, so we don't rescale on input method.
                sourceSize.height = parent.height;
                sourceSize.width = height * sourceSize.width / sourceSize.height;
            }
        }

        Component {
            id: attachPopovercomponent

            ActionSelectionPopover {
                id: attachPopover
                contentWidth: units.gu(22)
                focus: false
                z: 3
                delegate: ListItems.Standard {
                    iconFrame: false
                    iconSource: Qt.resolvedUrl(action.iconSource)
                    focus: false
                    text: action.text
                }
                actions: ActionList {
                    Action {
                        iconSource: "../images/ic_attach_gallery.png"
                        text: i18n.tr("Photo")
                        onTriggered: {
                            attachPopover.hide();
                            requestMedia(ContentType.Pictures);
                        }
                    }
                    Action {
                        iconSource: "../images/ic_attach_video.png"
                        text: i18n.tr("Video")
                        onTriggered: {
                            message.forceActiveFocus();
                            attachPopover.hide();
                            requestMedia(ContentType.Videos);
                        }
                    }
                    Action {
                        iconSource: "../images/ic_ab_doc.png"
                        text: i18n.tr("File")
                        onTriggered: {
                            message.forceActiveFocus();
                            attachPopover.hide();
                            requestMedia(ContentType.All);
                        }
                    }
                }
            }
        }

        Rectangle {
            id: addContactHeader
            anchors {
                top: parent.top
                left: parent.left
                right: parent.right
            }
            height: units.gu(5)
            color: "#44000000"
            visible: hasPhone && isForeign
            z: 1

            Text {
                id: addContactText
                anchors.centerIn: parent
                color: TelegramColors.white
                font {
                    pixelSize: FontUtils.sizeToPixels("small")
                    bold: true
                }
                text: i18n.tr("Add to contacts")
            }

            MouseArea {
                anchors.fill: parent
                onClicked: addContact()
            }
        }

        Rectangle {
            id: bottomRectangle
            anchors {
                left: parent.left
                right: parent.right
                bottom: parent.bottom
            }
            height: message.height + units.gu(2)
            z: 2

            color: "white"

            Label {
                id: messagePlacehoder
                anchors {
                    left: parent.left
                    right: parent.right
                    bottom: parent.bottom
                }
                height: parent.height
                verticalAlignment: Text.AlignVCenter
                horizontalAlignment: Text.AlignHCenter
                visible: pageIsSecret && secretChatState != 2
                color: "grey"
                fontSize: "medium"
                text: delegateUtils.getSecretChatState(secretChatState, true);
            }

            Label {
                id: audioRecordindLabel
                anchors {
                    left: parent.left
                    leftMargin: units.gu(2)
                    bottom: parent.bottom
                }
                height: parent.height
                verticalAlignment: Text.AlignVCenter
                visible: audioRecorder.recording
                color: "grey"
                fontSize: "medium"
                textFormat: Text.StyledText
                text: "<b><font color=\"black\" size=\"3\">"
                        + delegateUtils.formatTime(audioRecorder.duration, true)
                        + "</font></b>"
                        + i18n.tr(" ⇠ SLIDE TO CANCEL")
            }

            TextArea {
                id: message

                property int oldLength: 0

                anchors {
                    left: parent.left
                    right: sendButtonBox.left
                    bottom: parent.bottom
                    margins: units.gu(1)
                }

                // this value is to avoid letter and underline being cut off
                height: units.gu(4.3)
                enabled: isConnected
                visible: !messagePlacehoder.visible && !audioRecordindLabel.visible
                placeholderText: isConnected ? i18n.tr("Type message") : i18n.tr("Not connected.")
                inputMethodHints: Qt.ImhNone

                autoSize: true
                maximumLineCount: 4
                Keys.onReturnPressed: {
                    if (userSettings.contents.sendByEnter && isConnected) {
                        Qt.inputMethod.commit();
                        if (message.text.length === 0) return;
                        sendMessage(message.text);
                    } else {
                        event.accepted = false;
                    }
                }

                onLengthChanged: {
                    if (!typingTimer.running && length > oldLength) {
                        if (dialogModel.isSecret) {
                            telegramClient.messagesSetEncryptedTyping(dialogModel.chatId, true);
                        } else {
                            telegramClient.messagesSetTyping(inputPeer);
                        }
                        typingTimer.start();
                    }
                    oldLength = length;
                }
            }

            Item {
                id: sendButtonBox
                anchors {
                    top: parent.top
                    bottom: parent.bottom
                    right: parent.right
                    rightMargin: units.gu(2)
                }
                width: buttonsRow.width
                visible: !messagePlacehoder.visible

                Row {
                    id: buttonsRow
                    spacing: units.gu(2)
                    anchors.verticalCenter: sendButtonBox.verticalCenter

                    states: [
                        State {
                            name: "text"
                            when: message.inputMethodComposing || message.text.length > 0
                            PropertyChanges {
                                target: attachButton
                                width: 0
                                opacity: 0.0
                            }
                            PropertyChanges {
                                target: audioButton
                                width: 0
                                opacity: 0.0
                            }
                            PropertyChanges {
                                target: sendButton
                                width: units.gu(3.5)
                                opacity: 1.0
                            }
                        },
                        State {
                            name: "notext"
                            when: !message.inputMethodComposing && message.text.length === 0
                            PropertyChanges {
                                target: attachButton
                                width: units.gu(3.5)
                                opacity: 1.0
                            }
                            PropertyChanges {
                                target: audioButton
                                width: units.gu(3.5)
                                opacity: 1.0
                            }
                            PropertyChanges {
                                target: sendButton
                                width: 0
                                opacity: 0.0
                            }
                        }
                    ]

                    transitions: [
                        Transition {
                            PropertyAnimation {
                                target: attachButton
                                properties: "width, opacity"
                                duration: UbuntuAnimation.FastDuration
                            }
                            PropertyAnimation {
                                target: audioButton
                                properties: "width, opacity"
                                duration: UbuntuAnimation.FastDuration
                            }
                        }
                    ]

                    Image {
                        id: attachButton
                        height: units.gu(3.5)
                        fillMode: Image.PreserveAspectFit
                        focus: false
                        enabled: isConnected && message.text.length === 0
                        source: "../images/ic_ab_attach.png"
                        visible: message.visible

                        MouseArea {
                            anchors.fill: attachButton
                            onClicked: {
                                if (isConnected && actionsEnabled) {
                                    message.focus = false;
                                    Qt.inputMethod.hide();
                                    PopupUtils.open(attachPopovercomponent, attachButton)
                                }
                            }
                        }
                    }

                    Image {
                        id: audioButton
                        height: units.gu(3.5)
                        width: units.gu(3.5)
                        fillMode: Image.PreserveAspectFit
                        focus: false
                        enabled: isConnected && message.text.length === 0
                        source: audioRecorder.recording ? "../images/mic_pressed.png" : "../images/mic.png"

                        MouseArea {
                            id: audioMouseArea
                            anchors.fill: audioButton
                            property real lastX: -1
                            property bool isCancelling: false

                            onPressed: {
                                if (isConnected && actionsEnabled) {
                                    message.focus = false;
                                    Qt.inputMethod.hide();
                                    if (audioRecorder.recording) {
                                        audioRecorder.cancel();
                                    } else {
                                        audioRecorder.start();
                                    }
                                    lastX = mouse.x;
                                }
                            }

                            onPositionChanged: {
                                isCancelling = lastX - mouse.x > units.gu(10);
                                if (isCancelling) {
                                    audioRecorder.cancel();
                                }
                            }

                            onReleased: {
                                if (!isCancelling) {
                                    audioRecorder.stop();
                                }
                            }
                        }
                    }

                    Image {
                        id: sendButton
                        height: units.gu(3.5)
                        width: units.gu(3.5)
                        fillMode: Image.PreserveAspectFit
                        focus: false
                        enabled: isConnected
                        source: enabled ? "../images/ic_send.png" : "../images/ic_send_disabled.png"

                        MouseArea {
                            anchors.fill: sendButton
                            onClicked: {
                                Qt.inputMethod.commit();
                                if (message.text.length === 0) return;

                                sendMessage(message.text);
                                list.positionViewAtBeginning();
                            }
                        }
                    }
                }
            }
        }

        MultipleSelectionListView {
            id: list
            verticalLayoutDirection: ListView.BottomToTop
            anchors {
                top: parent.top
                left: parent.left
                bottom: bottomRectangle.top
                right: parent.right
            }
            header: Item {
                height: units.gu(2)
            }
            footer: Item {
                height: units.gu(1)
            }
            cacheBuffer: units.gu(10)*20
            highlightFollowsCurrentItem: false
            clip: true

            /*
            // Disabling section headers because of upstream bug.
            section.property: "sectionDate"
            section.criteria: ViewSection.FullString
            section.delegate: sectionHeading
            */

            listModel: messagesModel
            listDelegate: TelegramDelegate {
                id: delegate
                outgoing: model.out
                isSent: model.sent
                isRead: !model.unread
                isAction: model.isAction

                actionType: model.actionType
                actionTitle: model.actionTitle
                actionUser: model.actionUser
                actionThumbnail: isAction ? model.thumbnail : ""

                //FIXME: When section headers upstream bug is resolved revet this with section headers.
                isNewDay: (index === list.count) || !Time.areSameDay((messagesModel.get(index+1).date * 1000),model.date*1000)

                messageId: model.id
                ttl: model.ttl
                message: getMessageText(model.mediaType, outgoing)
                messageDate: model.date
                mediaType: model.mediaType
                senderColor: Avatar.getColor(model.fromId)
                senderId: model.fromId
                senderName: getSenderName()
                senderDisplayName: (!isChat || outgoing) ? "" : getSenderDisplayName()
                senderImage: {
                    if (isChat && !outgoing && !list.isInSelectionMode) {
                        return model.fromThumbnail;
                    }
                    return "";
                }
                time: Time.formatTimeOnly(i18n, model.date * 1000)
                thumbnail: isAction ? "" : model.thumbnail
                photo: model.photo
                video: model.video
                document: model.document
                documentFileName: model.documentFileName
                documentSize: model.documentSize
                audio: model.audio
                audioSize: model.audioSize
                audioDuration: model.audioDuration
                isDownloading: model.downloading || false
                progress: model.downloadedPercentage || 0

                isForwarded: model.fwdFromId > 0
                forwardedFromName: getForwardedFromName();

                onProfileImageClicked: {
                    openProfile(model.fromId);
                }

                onForwardedFromClicked: {
                    openProfile(model.fwdFromId);
                }

                leftSideActions: [
                    Action {
                        iconName: "delete"
                        text: i18n.tr("Delete")
                        visible: isConnected
                        onTriggered: {
                            // !isSent used to simulate secret message - only delete from the client.
                            telegramClient.messagesDeleteMessages([messageId], pageIsSecret || !isSent);
                        }
                    }
                ]
                rightSideActions: [
                     Action {
                        iconName: "reload"
                        text: i18n.tr("Resend")
                        visible: !isSent && !isAction
                        onTriggered: {
                            if (isPhoto) {
                                sendPhoto(photo);
                            } else if (isVideo) {
                                sendVideo(video);
                            } else {
                                sendMessage(message);
                            }
                        }
                    },
                    Action {
                        iconName: "edit-copy"
                        text: i18n.tr("Copy")
                        visible: !isPhoto && !isVideo && !isDocument
                        onTriggered: Clipboard.push(message)
                    },
                    Action {
                        iconName: "next"
                        text: i18n.tr("Forward")
                        visible: !pageIsSecret
                        onTriggered: forwardMessages([messageId])
                    }
                ]

                selected: list.isSelected(delegate)
                selectionMode: isAction ? false : list.isInSelectionMode

                onItemPressAndHold: {
                    list.startSelection()
                    if (list.isInSelectionMode) {
                        list.selectItem(delegate)
                    }
                }

                onItemClicked: {
                    if (list.isInSelectionMode && !isAction) {
                        if (selected) {
                            list.deselectItem(delegate)
                        } else {
                            list.selectItem(delegate)
                        }
                    }
                }

                locked: !isConnected || isAction

                function getSenderName() {
                    if (model.fromUserType === TLUser.TypeUserEmpty || model.fromUserType === TLUser.TypeUserDeleted) {
                        return i18n.tr("DELETED");
                    } else {
                        return model.fromFirstName
                    }
                }

                function getSenderDisplayName() {
                    if (model.fromUserType === TLUser.TypeUserEmpty || model.fromUserType === TLUser.TypeUserDeleted) {
                        return i18n.tr("DELETED");
                    } else {
                        return model.fromFirstName + " " + model.fromLastName;
                    }
                }

                function getForwardedFromName() {
                    if (model.fromUserType === TLUser.TypeUserEmpty || model.fromUserType === TLUser.TypeUserDeleted) {
                        return i18n.tr("DELETED");
                    } else if (model.fwdFromId > 0) {
                        return model.fwdFromFirstName + " " + model.fwdFromLastName;
                    }
                    return "";
                }

                function getMessageText(mediaType, outgoing) {
                    var type = delegateUtils.getMediaTypeString(mediaType);
                    switch (true) {
                        case type === "Text":
                        return model.text;
                        case type === "Photo":
                        return "";
                        case type === "Video":
                        return "";
                        case type === "Document":
                        return "";
                        case type === "Audio":
                        return "";
                        default:
                        return type + i18n.tr(" - unsupported message type. We're working on improving that! Meanwhile, make sure the app is up to date.");
                    }
                }
            }

            function deleteSelected() {
                var toDeletion = [];
                for (var i = 0; i < list.selectedItems.count; i++) {
                    toDeletion.push(list.selectedItems.get(i).model.id);
                }
                list.endSelection();
                telegramClient.messagesDeleteMessages(toDeletion, pageIsSecret);
            }

            function copySelected() {
                var message = "", item;
                for (var i = list.selectedItems.count - 1; i >= 0 ; i--) {
                    item = list.selectedItems.get(i);
                    if (delegateUtils.getMediaTypeString(item.model.mediaType) === "Text") {
                        message += item.model.text + "\n";
                    }
                }
                list.endSelection();
                Clipboard.push(message);
            }

            function forwardSelected() {
                var toForward = [];
                for (var i = list.selectedItems.count - 1; i >= 0 ; i--) {
                    toForward.push(list.selectedItems.get(i).model.id);
                }
                list.endSelection();
                forwardMessages(toForward);
            }

            Scrollbar {
                flickableItem: list
            }
        }

        Item {
            anchors.centerIn: parent
            width: pageLabel.width
            height: pageLabel.height

            Rectangle {
                anchors {
                    fill: pageLabel
                    margins: units.gu(-2)
                }
                radius: 5
                color: TelegramColors.chat_section
            }

            Label {
                id: pageLabel
                color: "white"
                fontSize: "medium"
                text: {
                    if (pageIsSecret) {
                        return i18n.tr("Secret chat with") + " " + title +
                        "\n\n" +
                        i18n.tr("Secret chats:") + "\n" +
                        i18n.tr("- Use end-to-end encryption") + "\n" +
                        i18n.tr("- Leave no trace on Telegram servers") + "\n" +
                        i18n.tr("- Do not allow forwarding");
                    } else {
                        return i18n.tr("No messages here yet...");
                    }
                }
            }

            visible: list.model.count === 0
            z: 1
        }

        DelegateUtils {
            id: delegateUtils
        }
    }

    MediaImport {
        id: mediaImporter

        onMediaReceived: {
            var filePath = String(mediaUrl).replace('file://', '');
            if (contentType === ContentType.Pictures) {
                dialogPage.sendPhoto(filePath);
            } else if (contentType === ContentType.Videos) {
                dialogPage.sendVideo(filePath);
            } else {
                dialogPage.sendDocument(filePath);
            }
            message.forceActiveFocus();
        }
    }

    AudioRecorder {
        id: audioRecorder

        onAudioRecorded: {
            console.log("audioRecorded", filePath, duration);
            // move to /audio/ and send
            telegramClient.sendAudio(chatId, filePath, duration/1000);
        }
    }

    Audio {
        id: audioPlayer
        property bool playing: playbackState === Audio.PlayingState
        property string filePath: ""

        source: filePath

        onStatusChanged: {
            // reset the position of the slider back to 0
            switch (status) {
            case Audio.Stalled:
            case Audio.EndOfMedia:
            case Audio.UnknownStatus:
                stop();
                filePath = "";
                break;
            }
        }
    }

    onAddContactsAnswer: {
        var contactAdded = false;
        for (var i = 0; i < contactIds.length; i++) {
            var contactId = contactIds[i];
            if (contactId === dialogModel.chatId) {
                dialogModel.refreshDialogDetails(chatId);
                contactAdded = true;
                break;
            }
        }
        if (!contactAdded) {
            console.warn("Contacts added, but requested contact not.")
        }
    }

    onSecretChatCreated: {
        secretChatState = 2;
    }

    onSecretChatDiscarded: {
        secretChatState = 3;
    }

    function requestMedia(mediaType) {
        Qt.inputMethod.hide();
        mediaImporter.contentType = mediaType;
        mediaImporter.requestMedia();
    }

    function sendMessage(text) {
        if (text.length === 0) return;
        if (dialogModel.isSecret) {
            telegramClient.sendSecretChatMessage(dialogModel.chatId, text, dialogModel.ttl);
        } else {
            telegramClient.messagesSendMessage(inputPeer, text);
        }
        message.text = "";
    }

    Timer {
        id: typingTimer
        interval: 5000
        repeat: false
        triggeredOnStart: false
    }

    Timer {
        id: sendPhotoTimer

        property int attempts: 1
        property string photo: ""

        function send(photoPath) {
            stop();
            photo = photoPath;
            attempts = 0;
            restart();
        }

        interval: 750
        repeat: true
        onTriggered: {
            if (telegramClient.connected) {
                console.log("Connected, sending photo.");
                if (pageIsSecret) {
                    telegramClient.sendSecretChatPhoto(dialogModel.chatId, photo, dialogModel.ttl);
                } else {
                    telegramClient.messagesSendPhoto(inputPeer, photo);
                }
                stop();
            } else {
                attempts += 1;
                if (attempts >= 4) {
                    console.log("Picture not sent, giving up!");
                    stop();
                } else {
                    console.log("Picture not sent, will re-try. (" + attempts + ")");
                }
            }
        }
    }

    function sendPhoto(photo) {
        if (photo.length === 0) return;
        sendPhotoTimer.send(photo);
        // TODO Instead, this should be handled by qml-plugin. Queue the upload.
        // telegramClient.messagesSendPhoto(inputPeer, photo);
    }

    Timer {
        id: sendVideoTimer

        property int attempts: 1
        property string video: ""

        function send(videoPath) {
            stop();
            video = videoPath;
            attempts = 0;
            restart();
        }

        interval: 750
        repeat: true
        onTriggered: {
            if (telegramClient.connected) {
                console.log("Connected, sending video.");
                if (pageIsSecret) {
                    telegramClient.sendSecretChatVideo(dialogModel.chatId, video, dialogModel.ttl);
                } else {
                    telegramClient.sendVideo(dialogModel.chatId, video);
                }
                stop();
            } else {
                attempts += 1;
                if (attempts >= 4) {
                    console.log("Video not sent, giving up!");
                    stop();
                } else {
                    console.log("Video not sent, will re-try. (" + attempts + ")");
                }
            }
        }
    }

    function sendVideo(video) {
        if (video.length === 0) return;
        sendVideoTimer.send(video);
    }

    Timer {
        id: sendDocumentTimer

        property int attempts: 1
        property string document: ""

        function send(documentPath) {
            stop();
            document = documentPath;
            attempts = 0;
            restart();
        }

        interval: 750
        repeat: true
        onTriggered: {
            if (telegramClient.connected) {
                console.log("Connected, sending document.");
                if (pageIsSecret) {
                    telegramClient.sendSecretChatDocument(dialogModel.chatId, document, dialogModel.ttl);
                } else {
                    telegramClient.messagesSendDocument(inputPeer, document);
                }
                stop();
            } else {
                attempts += 1;
                if (attempts >= 4) {
                    console.log("Picture not sent, giving up!");
                    stop();
                } else {
                    console.log("Picture not sent, will re-try. (" + attempts + ")");
                }
            }
        }
    }

    function sendDocument(document) {
        if (document.length === 0) return;
        sendDocumentTimer.send(document);
    }

    function showClearHistoryDialog() {
        PopupUtils.open(Qt.resolvedUrl("dialogs/ConfirmationDialog.qml"), dialogPage, {
                text: i18n.tr("Are you sure you want to clear history?"),
                onAccept: function() {
                    console.log("Deleting chat history.");
                    if (pageIsSecret) {
                        telegramClient.deleteSecretChatHistory(dialogModel.chatId, false);
                    } else {
                        telegramClient.deleteChatHistory(inputPeer, 0, false);
                    }
                }
            }
        );
    }

    function showDeleteChatDialog() {
        PopupUtils.open(Qt.resolvedUrl("dialogs/ConfirmationDialog.qml"), dialogPage, {
                text: i18n.tr("Are you sure you want to delete this chat?"),
                onAccept: function() {
                    console.log("Deleting chat.");
                    if (pageIsSecret) {
                        telegramClient.discardSecretChat(dialogId);
                        telegramClient.deleteSecretChatHistory(dialogModel.chatId, true);
                    } else {
                        telegramClient.deleteChatHistory(inputPeer, 0, true);
                    }
                }
            }
        );
    }

    function showSelfDestructDialog() {
        PopupUtils.open(Qt.resolvedUrl("dialogs/SelfDestructDialog.qml"), dialogPage, {
                chatId: chatId
            }
        );
    }

    function addContact() {
        if (isChat) {
            console.warn("Adding single contact supported only.");
            return;
        }
        showAddContactPage(false, dialogModel.chatId);
    }

    function addMember() {
        if (dialogModel.peerType !== TLInputPeer.TypeInputPeerChat) {
            console.warn("Trying to add member to a non-group chat!");
            return;
        }

        var properties = { addToGroupMode: true,
                           groupChatId: dialogModel.chatId,
                           groupChatTitle: dialogModel.title };
        pageStack.push(contactsPage, properties);
    }
}
