import QtQuick 2.0
import Telegram 0.1
import Ubuntu.Components 1.1
import Ubuntu.Components.ListItems 1.0 as ListItem
import Ubuntu.Contacts 0.1
import Ubuntu.Content 1.0

import "../components"
import "../js/avatar.js" as Avatar
import "../js/time.js" as Time

TelegramPage {
    id: groupProfilePage

    property int chatId

    signal bigPhotoUpdated(var id, string bigPhotoUrl);

    GroupModel {
        id: groupModel

        onMutedChanged: checkbox.toggleSilently(muted)
    }

    GroupMembersModel {
        id: membersModel
    }

    function addMember() {
        var properties = {
            addToGroupMode: true,
            groupChatId: groupModel.chatId,
            groupChatTitle: groupModel.title
        };
        pageStack.push(contactsPage, properties);
    }

    pageTitle: i18n.tr("Group Info")

    head.actions: [
        Action {
            iconName: "contact-new"
            text: i18n.tr("Add member")
            enabled: isConnected
            onTriggered: addMember()
        },
        Action {
            iconName: "stock_image"
            text: i18n.tr("Change photo")
            onTriggered: {
                Qt.inputMethod.hide();
                requestPicture();
            }
        }

    ]

    body: Item {
        anchors.fill: parent

        Image {
            id: photoImage
            anchors {
                top: parent.top
                topMargin: units.gu(2)
                left: parent.left
                leftMargin: units.gu(2)
            }
            width: source == "" || source === undefined ? 0 : units.gu(9)
            height: width
            asynchronous: true
            fillMode: Image.PreserveAspectCrop
            source: groupModel.thumbnail !== "" ? groupModel.thumbnail
                                            : "../images/" + Avatar.getAvatar(groupModel.chatId, true)

            MouseArea {
                anchors.fill: parent
                onClicked: {
                    if (groupModel.thumbnail) {
                        var properties = { "title" : groupModel.title, "chatId" : chatId  };
                        if (groupModel.photo) {
                            properties.photoPreviewSource = groupModel.photo;
                        } else {
                            properties.photoPreviewSource = groupModel.thumbnail;
                            telegramClient.downloadChatBigPhoto(groupModel.chatId);
                        }
                        pageStack.push(previewPage, properties);
                    }
                }
            }
        }

        TelegramSubtitledListItem {
            id: detailsItem
            anchors {
                left: photoImage.right
                right: parent.right
                verticalCenter: photoImage.verticalCenter
            }
            highlightWhenPressed: false
            showDivider: false

            title: groupModel.title
            titleIsBold: true
            titleMaxLineCount: 2
            subtitle: groupModel.count + " " + i18n.tr("members")
        }

        ListItem.Standard {
            id: notificationsItem
            anchors.top: photoImage.bottom
            anchors.topMargin: units.gu(1)

            text: i18n.tr("Notifications")
            showDivider: true

            Switch {
                // avoid sending signal to server when reflecting server state
                property bool override: true

                id: checkbox
                anchors {
                    right: parent.right
                    rightMargin: units.gu(2)
                    verticalCenter: parent.verticalCenter
                }

                onCheckedChanged: {
                    if (override) return;

                    if (checked) {
                        telegramClient.unmute(chatId);
                    } else {
                        telegramClient.mute(chatId);
                    }
                }

                Component.onCompleted: {
                    checkbox.checked = !groupModel.muted;
                    override = false;
                }

                function toggleSilently(muted) {
                    override = true;
                    checked = !muted;
                    override = false;
                }
            }
        }

/*
// Intentionally left commented.
            Component {
                id: photoActionsComponent

                Popover {
                    id: popover
                    Column {
                        id: containerLayout
                        anchors {
                            left: parent.left
                            top: parent.top
                            right: parent.right
                        }
                        ListItem.Header { text: i18n.tr("Group photo") }
                        ListItem.Standard { text: i18n.tr("Open photo") }
                        ListItem.Standard { text: i18n.tr("From camera") }
                        ListItem.Standard { text: i18n.tr("From gallery") }
                        ListItem.Standard { text: i18n.tr("Delete photo") }
                    }
                }
            }

            MouseArea {
                anchors.fill: parent
                onClicked: PopupUtils.open(photoActionsComponent, photoImage)
            }
*/

        ListView {
            id: members
            anchors {
                top: notificationsItem.bottom
                left: parent.left
                bottom: parent.bottom
                right: parent.right
            }
            footer: Item {
                height: units.gu(1)
            }
            highlightFollowsCurrentItem: false
            clip: true

            model: membersModel
            delegate: TelegramContactsListItem {
                id: delegate

                userId: model.id
                photo: model.photo
                title: model.name
                isOnline: model.online
                lastSeen: model.lastSeenOnline

                onItemClicked: {
                    mouse.accepted = true;
                    if (model.id !== telegramClient.ourId) {
                        openProfile(model.id);
                    }
                }

                leftSideActions: [
                    Action {
                        iconName: "remove"
                        text: i18n.tr("Remove from group")
                        visible: isConnected && isRemovable
                        onTriggered: {
                            telegramClient.deleteChatUser(groupModel.chatId, model.id);
                            if (model.id === telegramClient.ourId &&
                                    pageStack.currentPage.objectName === "groupProfilePage") {
                                pageStack.pop();
                            }
                        }
                    }
                ]

                rightSideActions: [
                    Action {
                        iconName: "info"
                        text: i18n.tr("Info")
                        onTriggered: openProfile(model.id);
                    }
                ]
            }
        }
    }

    Component.onCompleted: {
        telegramClient.chatBigPhotoUpdated.connect(groupProfilePage.bigPhotoUpdated);
        membersModel.setup(telegramClient, chatId);
        groupModel.setup(telegramClient, chatId);
    }

    Component.onDestruction: {
        telegramClient.chatBigPhotoUpdated.disconnect(groupProfilePage.bigPhotoUpdated);
    }

    onBigPhotoUpdated: {
        if (id === groupModel.chatId) {
            // refresh model to fill the big photo from database.
            groupModel.refreshGroupDetails(id);
        }
    }

    MediaImport {
        id: pictureImporter
        contentType: ContentType.Pictures

        onMediaReceived: {
            var filePath = String(mediaUrl).replace('file://', '')
            if (filePath.length > 0) {
                groupProfilePage.uploadPhoto(filePath);
            }
        }
    }

    Timer {
        id: editPhotoTimer

        property int attempts: 1
        property string photo: ""

        function upload(photoPath) {
            stop();
            photo = photoPath;
            attempts = 0;
            restart();
        }

        interval: 750
        repeat: true
        onTriggered: {
            if (telegramClient.connected) {
                console.log("Connected, uploading profile photo.");
                telegramClient.editChatPhoto(chatId, photo);
                stop();
            } else {
                attempts += 1;
                if (attempts >= 4) {
                    console.log("Picture not uploaded, giving up!");
                    stop();
                } else {
                    console.log("Picture not uploaded, will re-try. (" + attempts + ")");
                }
            }
        }
    }

    function requestPicture() {
        Qt.inputMethod.hide();
        pictureImporter.requestMedia();
    }

    function uploadPhoto(photo) {
        editPhotoTimer.upload(photo);
        // TODO Instead, this should be handled by qml-plugin. Queue the upload.
    }
}
