import QtQuick 2.0
import QtQuick.Layouts 1.1
import Telegram 0.1
import Ubuntu.Components 1.1
import Ubuntu.Components.ListItems 1.0 as ListItem
import "../components"
import "../components/TelegramColors.js" as TelegramColors
import "../js/avatar.js" as Avatar
import "../js/time.js" as Time

TelegramPage {
    id: profilePage

    property int dialogId: 0
    property bool userExists: true

    ProfileModel {
        id: profileModel

        onIsMutedChanged: notificationsCheckbox.show(!isMuted)

        onIsBlockedChanged: blockedCheckbox.show(isBlocked)
    }

    function getUserInfo() {
        telegramClient.getUser(profileModel.userId);
    }

    pageTitle: i18n.tr("Contact Info")

    body: Item {
        anchors.fill: parent

        Image {
            id: photoImage
            anchors {
                top: parent.top
                topMargin: units.gu(2)
                left: parent.left
                leftMargin: units.gu(2)
            }
            width: source == "" || source === undefined ? 0 : units.gu(9)
            height: width
            asynchronous: true
            fillMode: Image.PreserveAspectCrop
            source: profileModel.thumbnail !== "" ? profileModel.thumbnail
                    : "../images/" + Avatar.getAvatar(profileModel.userId, profileModel.isChat)

            MouseArea {
                anchors.fill: parent
                onClicked: {
                    if (profileModel.thumbnail) {
                        var properties = {
                            "title" : profileModel.title,
                            "chatId" : profileModel.dialogId
                        };
                        if (profileModel.photo !== "") {
                            properties.photoPreviewSource = profileModel.photo;
                        } else {
                            properties.photoPreviewSource = profileModel.thumbnail;
                            profileModel.downloadPhoto();
                        }
                        pageStack.push(previewPage, properties);
                    }
                }
            }
        }

        TelegramSubtitledListItem {
            id: detailsItem
            anchors {
                left: photoImage.right
                right: parent.right
                verticalCenter: photoImage.verticalCenter
            }
            highlightWhenPressed: false
            showDivider: false

            title: profileModel.title
            titleIsBold: true
            titleMaxLineCount: 2
            subtitle: {
                if (profileModel.isChat) {
                    return groupModel.count + " " + i18n.tr("members")
                } else {
                    if (profileModel.isOnline) {
                        return i18n.tr("online");
                    } else if (profileModel.lastSeen > 0) {
                        var lastSeenString = Time.formatLastSeen(i18n, profileModel.lastSeen * 1000);
                        return i18n.tr("last seen %1").arg(lastSeenString);
                    } else {
                        return "";
                    }
                }
            }
        }

        ListItem.Header {
            id: phoneHeader
            anchors {
                top: photoImage.bottom
                topMargin: units.gu(1)
            }
            text: i18n.tr("Phone Number")
        }

        RowLayout {
            id: phoneRow
            anchors {
                top: phoneHeader.bottom
                left: parent.left
                leftMargin: units.gu(2)
                right: parent.right
                rightMargin: units.gu(2)
            }
            height: units.gu(5)

            Label {
                id: phone
                text: profileModel.hasPhone ? "+" + profileModel.phone : i18n.tr("Unknown")
                Layout.fillWidth: true
            }

            AbstractButton {
                id: callAction
                width: units.gu(5)
                height: width
                visible: profileModel.hasPhone
                Icon {
                    width: units.gu(4)
                    height: width
                    anchors.centerIn: parent
                    source: Qt.resolvedUrl("../images/call.png")
                }
                onClicked: {
                    Qt.openUrlExternally("tel:///" + phone.text)
                }
            }
        }

        ListItem.Header {
            id: contactHeader
            text: i18n.tr("Settings")
            anchors.top: phoneRow.bottom
        }

        TLInputPeer {
            id: inputPeer
            userId: profileModel.userId
        }

        ListItem.Standard {
            id: notificationsItem
            anchors.top: contactHeader.bottom

            text: i18n.tr("Notifications")
            showDivider: true

            onClicked: notificationsCheckbox.checked = !notificationsCheckbox.checked

            Switch {
                // avoid sending signal to server when reflecting server state
                property bool override: true

                id: notificationsCheckbox
                anchors {
                    right: parent.right
                    rightMargin: units.gu(2)
                    verticalCenter: parent.verticalCenter
                }

                onCheckedChanged: {
                    if (override) return;

                    if (checked) {
                        telegramClient.unmute(profileModel.dialogId);
                    } else {
                        telegramClient.mute(profileModel.dialogId);
                    }
                }

                Component.onCompleted: {
                    notificationsCheckbox.checked = !profileModel.isMuted;
                    override = false;
                }

                function show(isMuted) {
                    override = true;
                    checked = isMuted;
                    override = false;
                }
            }
        }

        ListItem.Standard {
            id: blockItem
            anchors.top: notificationsItem.bottom

            text: i18n.tr("Block user")
            showDivider: true

            onClicked: blockedCheckbox.checked = !blockedCheckbox.checked

            Switch {
                // avoid sending signal to server when reflecting server state
                property bool override: true

                id: blockedCheckbox
                anchors {
                    right: parent.right
                    rightMargin: units.gu(2)
                    verticalCenter: parent.verticalCenter
                }

                onCheckedChanged: {
                    if (override) return;

                    if (checked) {
                        telegramClient.blockContact(inputPeer);
                    } else {
                        telegramClient.unblockContact(inputPeer);
                    }
                }

                Component.onCompleted: {
                    blockedCheckbox.checked = profileModel.isBlocked;
                    override = false;
                }

                function show(isBlocked) {
                    override = true;
                    checked = isBlocked;
                    override = false;
                }
            }
        }

        TelegramButton {
            id: msgAction
            anchors {
                left: parent.left
                right: parent.right
                bottom: startSecretChat.visible ? startSecretChat.top : parent.bottom
                margins: units.gu(2)
            }
            visible: profileModel.exists
            text: i18n.tr("Start chat")
            color: TelegramColors.grey
            onClicked: {
                openChatById(profileModel.userId);
            }
        }

        TelegramButton {
            id: startSecretChat
            anchors {
                left: parent.left
                right: parent.right
                bottom: parent.bottom
                margins: units.gu(2)
            }
            text: i18n.tr("Start secret chat")
            enabled: isConnected
            visible: profileModel.exists && !profileModel.isSecret
            onClicked: {
                var userId = profileModel.userId;
                backToDialogsPage();
                telegramClient.createSecretChat(userId);
            }
        }
    }

    Component.onCompleted: {
        profileModel.setup(telegramClient, dialogId);
    }
}
