import QtQuick 2.0
import Ubuntu.Components 1.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import Ubuntu.Components.Popups 0.1
import Ubuntu.Content 1.0
import Telegram 0.1

import "../components"
import "../js/version.js" as Version
import "../js/avatar.js" as Avatar
import "../js/time.js" as Time

TelegramPage {
    id: page
    pageTitle: i18n.tr("Settings");

    function getSubtitle(online, lastSeenOnline) {
       if (online) {
            return i18n.tr("online");
        } else {
           if (lastSeenOnline !== "") {
               return i18n.tr("last seen %1").arg(Time.formatLastSeen(i18n, lastSeenOnline * 1000));
           } else {
               return "";
           }
        }
    }

    function requestPicture() {
        Qt.inputMethod.hide();
        pictureImporter.requestMedia();
    }

    function showEditProfilePage() {
        var properties = { "firstName": profileModel.firstName, "lastName": profileModel.lastName };
        pageStack.push(editProfilePage, properties);
    }

    function uploadPhoto(photo) {
        if (photo.length === 0) return;
        editPhotoTimer.upload(photo);
        // TODO Instead, this should be handled by qml-plugin. Queue the upload.
        // telegramClient.messagesSendPhoto(inputPeer, photo);
    }

    property list<Action> defaultActions: [
        Action {
            iconName: "edit"
            text: i18n.tr("Edit name")
            onTriggered: {
                Qt.inputMethod.hide();
                showEditProfilePage();
            }
        },
        Action {
            iconName: "stock_image"
            text: i18n.tr("Change photo")
            onTriggered: {
                Qt.inputMethod.hide();
                requestPicture();
            }
        }
    ]

    head.actions: defaultActions

    MediaImport {
        id: pictureImporter
        contentType: ContentType.Pictures

        onMediaReceived: {
            var filePath = String(mediaUrl).replace('file://', '')
            page.uploadPhoto(filePath);
        }
    }

    ProfileModel {
        id: profileModel
    }

    VisualItemModel {
        id: model

        ListItem.Header {
            text: i18n.tr("Your Phone Number")
        }

        ListItem.Standard {
            text: telegramClient.phoneNumber
            showDivider: false
        }

        ListItem.Header {
            text: i18n.tr("Settings")
        }

        ListItem.Standard {
            text: i18n.tr("Notifications")
            height: canConfigurePush() ? implicitHeight : 0
            enabled: isConnected
            progression: true
            onClicked: openPushDialog()

            function canConfigurePush() {
                if (pushClientLoader.status === Loader.Ready) {
                    return pushClient.registered === false;
                } else {
                    return false;
                }
            }
        }

        ListItem.Standard {
            text: i18n.tr("Blocked users")
            showDivider: false
            enabled: page.isConnected
            onTriggered: pageStack.push(blockedUsersPage)
        }

        ListItem.Standard {
            text: i18n.tr("Terminate All Other Sessions")
            showDivider: false
            enabled: page.isConnected
            onClicked: PopupUtils.open(terminateSessionsDialogComponent)

            Component {
                id: terminateSessionsDialogComponent
                Dialog {
                    id: terminateSessionsDialog
                    title: i18n.tr("Telegram")
                    text: i18n.tr("Are you sure you want to terminate all other sessions?")
                    Button {
                        text: i18n.tr("OK")
                        color: UbuntuColors.orange
                        onClicked: {
                            console.log("Terminating all other sessions.");
                            PopupUtils.close(terminateSessionsDialog)
                            telegramClient.authResetAuthorizations();
                        }
                    }
                    Button {
                        text: i18n.tr("Cancel")
                        onClicked: PopupUtils.close(terminateSessionsDialog)
                    }
                }
            }
        }

        ListItem.Header {
            text: i18n.tr("Messages")
        }

        ListItem.ItemSelector {
            text: i18n.tr("Messages Text Size")
            showDivider: false

            model: [
                i18n.tr("Small"),
                i18n.tr("Medium"),
                i18n.tr("Large")
            ]

            selectedIndex: {
                console.log(userSettings.contents.fontSize)
                switch (userSettings.contents.fontSize) {
                    case "small": return 0;  // no-i18n
                    case "medium": return 1; // no-i18n
                    case "large": return 2;  // no-i18n
                }
            }

            onDelegateClicked: {
                switch (index) {
                    case 0: {
                        userSettings.set("fontSize", "small")
                        break;
                    }
                    case 1: {
                        userSettings.set("fontSize", "medium")
                        break;
                    }
                    case 2: {
                        userSettings.set("fontSize", "large")
                        break;
                    }
                }
            }
        }

        ListItem.Standard {
            text: i18n.tr("Send by Enter")
            showDivider: false

            Switch {
                id: checkbox
                checked: userSettings.contents.sendByEnter
                anchors {
                    right: parent.right
                    rightMargin: units.gu(2)
                    verticalCenter: parent.verticalCenter
                }

                onCheckedChanged: userSettings.set("sendByEnter", checked)
            }
        }

        ListItem.Header {
            text: i18n.tr("Support")
        }

        ListItem.Standard {
            text: i18n.tr("Telegram FAQ")
            onClicked: Qt.openUrlExternally("https://telegram.org/faq");
        }

        ListItem.Standard {
            showDivider: false
            text: i18n.tr("Ask a Question")
            // Can we go the Telegram route, or point people to askubuntu.com?
            onClicked: Qt.openUrlExternally("http://askubuntu.com/search?q=telegram");
        }

        TelegramButton {
            id: logOut
            anchors {
                left: parent.left
                right: parent.right
                margins: units.gu(2)
            }
            text: i18n.tr("Log Out")
            enabled: page.isConnected
            onClicked: PopupUtils.open(logOutDialogComponent)

            Component {
                id: logOutDialogComponent
                Dialog {
                    id: logOutDialog
                    title: i18n.tr("Telegram")
                    text: i18n.tr("Are you sure you want to log out?")
                    Button {
                        text: i18n.tr("OK")
                        color: UbuntuColors.orange
                        onClicked: {
                            console.log("Logging out.");
                            // Calling authLogOut here doesn't work.
                            // This seems to be an SDK bug.
                            logOutTimer.start();
                            PopupUtils.close(logOutDialog);
                        }
                    }
                    Button {
                        text: i18n.tr("Cancel")
                        onClicked: PopupUtils.close(logOutDialog)
                    }
                }
            }
        }

        ListItem.SingleControl {
            showDivider: false
            control: Label {
                id: version
                width: parent.width - units.gu(2)
                height: units.gu(4)
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                // TODO Add app version here below.
                // TRANSLATORS: the argument refers to the version of the app
                text: i18n.tr("Telegram for Ubuntu %1").arg(getVersionString());

                function getVersionString() {
                    var v = Version.version.split(".");
                    var major = v[0];
                    var minor = v[1];
                    var patch = v[2];
                    var revision = v.slice(3, v.length).join(".");

                    return "v" + major + "." + minor + "." + patch + " (" + revision + ")";
                }
            }
        }
    }

    body: Item {
        anchors.fill: parent

        Image {
            id: photoImage
            anchors {
                top: parent.top
                topMargin: units.gu(2)
                left: parent.left
                leftMargin: units.gu(2)
            }
            width: source == "" || source === undefined ? 0 : units.gu(9)
            height: width
            asynchronous: true
            fillMode: Image.PreserveAspectCrop
            source: profileModel.thumbnail !== "" ? profileModel.thumbnail
                                            : "../images/" + Avatar.getAvatar(profileModel.userId, false)

            MouseArea {
                anchors.fill: parent
                onClicked: {
                    if (profileModel.thumbnail) {
                        var properties = { "title" : profileModel.title };
                        if (profileModel.photo !== "") {
                            properties.photoPreviewSource = profileModel.photo;
                        } else {
                            properties.photoPreviewSource = profileModel.thumbnail;
                            profileModel.downloadPhoto();
                        }
                        pageStack.push(previewPage, properties);
                    }
                }
            }

        }

        TelegramSubtitledListItem {
            id: detailsItem
            anchors {
                left: photoImage.right
                right: parent.right
                verticalCenter: photoImage.verticalCenter
            }
            highlightWhenPressed: false
            showDivider: false

            title: profileModel.title
            titleIsBold: true
            titleMaxLineCount: 2
            subtitle: getSubtitle(profileModel.isOnline, profileModel.lastSeen)
        }

        ListView {
            anchors {
                topMargin: units.gu(2)
                top: photoImage.bottom
                left: parent.left
                bottom: parent.bottom
                right: parent.right
            }
            clip: true
            model: model
        }
    }

    Timer {
        id: logOutTimer
        interval: 300
        repeat: false

        onTriggered: {
            pushClient.unregisterFromPush();

            userData.reset();
            telegramClient.authLogOut();
        }
    }

    Timer {
        id: editPhotoTimer

        property int attempts: 1
        property string photo: ""

        function upload(photoPath) {
            stop();
            photo = photoPath;
            attempts = 0;
            restart();
        }

        interval: 750
        repeat: true
        onTriggered: {
            if (telegramClient.connected) {
                console.log("Connected, uploading profile photo.");
                telegramClient.uploadProfilePhoto(photo);
                stop();
            } else {
                attempts += 1;
                if (attempts >= 4) {
                    console.log("Picture not uploaded, giving up!");
                    stop();
                } else {
                    console.log("Picture not uploaded, will re-try. (" + attempts + ")");
                }
            }
        }
    }

    Component.onCompleted: {
        profileModel.setup(telegramClient, telegramClient.ourId);
    }
}
